"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.guard = void 0;
const symbols_1 = require("./symbols");
function guard(name, fn, handleOptions) {
    const fnWithError = (val, path = []) => {
        const valid = fn(val, {});
        return {
            valid,
            errors: valid
                ? []
                : [
                    {
                        message: `vality.${name}.type`,
                        path,
                        options: {},
                        val,
                    },
                ],
        };
    };
    return Object.assign((options) => {
        const fnWithErrorAndOptions = (val, path = []) => {
            const valid = fn(val, options);
            if (!valid)
                return {
                    valid,
                    errors: [
                        {
                            message: `vality.${name}.type`,
                            path,
                            options,
                            val,
                        },
                    ],
                };
            if (handleOptions === undefined)
                return { valid: true, errors: [] };
            const keysWithError = Object.keys(options).filter(k => !handleOptions[k](val, options[k], options));
            return {
                valid: keysWithError.length === 0,
                errors: keysWithError.map(k => ({
                    message: `vality.${name}.options.${k}`,
                    path,
                    options,
                    val,
                })),
            };
        };
        return Object.assign({
            [symbols_1._validate]: fnWithErrorAndOptions,
            // This is ugly, but the only way to ensure [_type] keeps its type
            [symbols_1._type]: undefined,
            [symbols_1._options]: options,
        });
    }, {
        [symbols_1._validate]: fnWithError,
        [symbols_1._type]: undefined,
        [symbols_1._options]: {},
    });
}
exports.guard = guard;
