"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const valit_1 = require("./valit");
const vality_1 = require("./vality");
vality_1.vality.array = (0, valit_1.valit)("array", e => (val, path, options) => {
    if (!Array.isArray(val))
        return { valid: false, errors: [{ message: "vality.array.type", path, options, val }] };
    const fn = (0, utils_1.enyToGuardFn)(e);
    const errors = (0, utils_1.flat)(val.map((_, i) => i).map(i => fn(val[i], [...path, i]).errors));
    return { valid: errors.length === 0, errors };
}, {
    minLength: (val, o) => val.length >= o,
    maxLength: (val, o) => val.length <= o,
});
vality_1.vality.object = (0, valit_1.valit)("object", e => (val, path, options) => {
    if (typeof val !== "object" || val === null)
        return { valid: false, errors: [{ message: "vality.object.type", path, options, val }] };
    const errors = [];
    for (const k in e) {
        // We can do this assertion here, since in the worst case, we'll get undefined, which is what we want to
        const res = (0, utils_1.enyToGuardFn)(e[k])(val[k], [...path, k]);
        errors.push(...res.errors);
        if (!res.valid && options.bail)
            break;
    }
    return { valid: errors.length === 0, errors };
});
vality_1.vality.optional = (0, valit_1.valit)("optional", e => (val, path) => {
    if (val === undefined)
        return { valid: true, errors: [] };
    return (0, utils_1.enyToGuardFn)(e)(val, path);
});
vality_1.vality.enum = (0, valit_1.valit)("enum", (...es) => (val, path, options) => {
    const valid = es.some(e => (0, utils_1.enyToGuardFn)(e)(val, path).valid);
    return { valid, errors: valid ? [] : [{ message: "vality.enum.type", path, options, val }] };
});
vality_1.vality.tuple = (0, valit_1.valit)("tuple", (...es) => (val, path, options) => {
    if (!Array.isArray(val) || val.length !== es.length)
        return { valid: false, errors: [{ message: "vality.tuple.type", path, options, val }] };
    const errors = (0, utils_1.flat)(val.map((_, i) => i).map(i => (0, utils_1.enyToGuardFn)(es[i])(val[i], [...path, i]).errors));
    return { valid: errors.length === 0, errors };
});
